<?php
namespace Drupal\Tests\aws_cloud\Functional\Ec2;

// Updated by yas 2016/09/07
// Updated by yas 2016/06/24
// Updated by yas 2016/06/23
// Updated by yas 2016/06/02
// Updated by yas 2016/05/31
// Updated by yas 2016/05/29
// Updated by yas 2016/05/25
// Updated by yas 2016/05/24
// Updated by yas 2016/05/23
// Updated by yas 2016/05/22
// Created by yas 2016/05/21.

use Drupal\Component\Utility\Random;
use Drupal\Component\Serialization\Yaml;

use Drupal\Tests\aws_cloud\Functional\Utils;

define('AWS_CLOUD_INSTANCE_REPEAT_COUNT', 3);

/**
 * Tests AWS Cloud Instance.
 *
 * @group AWS Cloud
 */
class InstanceTest extends AwsCloudTestCase {

  protected function getPermissions() {
    return [
      'add aws cloud instance',
      'list aws cloud instance',
      'edit own aws cloud instance',
      'delete own aws cloud instance',

      'list cloud server template',
      'launch server template',
    ];
  }

  protected function getMockDataTemplateVars() {
    $public_ip = Utils::getRandomPublicIp();
    $private_ip = Utils::getRandomPrivateIp();
    $regions = ['us-west-1', 'us-west-2'];
    $region = $regions[array_rand($regions)];

    return [
      'owner_id' => rand(100000000000, 999999999999), //12 digits
      'reservation_id' => 'r-' . $this->random->name(17, TRUE),
      'group_name' => $this->random->name(8, TRUE),
      'host_id' => $this->random->name(8, TRUE),
      'affinity' => $this->random->name(8, TRUE),
      'launch_time' => date('c'),
      'security_group_id' => 'sg-' . $this->random->name(8, TRUE),
      'security_group_name' => $this->random->name(10, TRUE),
      'public_dns_name' => Utils::getPublicDns($region, $public_ip),
      'public_ip_address' => $public_ip,
      'private_dns_name' => Utils::getPrivateDns($region, $private_ip),
      'private_ip_address' => $private_ip,
      'vpc_id' => 'vpc-' . $this->random->name(8, TRUE),
      'subnet_id' => 'subnet-' . $this->random->name(8, TRUE),
      'image_id' => 'ami-' . $this->random->name(8, TRUE),
      'reason' => $this->random->string(16, TRUE),
      'instance_id' => 'i-' . $this->random->name(17, TRUE),
    ];
  }

  /**
   * Tests CRUD for instance information.
   */
  public function testInstance() {
    $cloud_context = $this->cloud_context;

    $this->createServerTemplate();

    // List Instance for Amazon EC2.
    $this->drupalGet("/clouds/aws_cloud/$cloud_context/instance");
    $this->assertResponse(200, t('HTTP 200: List | Instance'));
    $this->assertNoText(t('Notice'), t('List | Make sure w/o Notice'));
    $this->assertNoText(t('warning'), t('List | Make sure w/o Warnings'));

    // Launch a new Instance.
    $add = $this->createInstanceTestData();
    for ($i = 0; $i < AWS_CLOUD_INSTANCE_REPEAT_COUNT; $i++) {
      $num = $i + 1;
      $this->addInstanceMockData($add[$i]['name'], $add[$i]['key_pair_name']);
      $this->drupalPostForm("/clouds/design/server_template/$cloud_context/1/launch",
                            [],
                            t('Launch'));
      $this->assertResponse(200, t('HTTP 200: Launch | A New Cloud Instance #@num', ['@num' => $num]));
      $this->assertNoText(t('Notice'), t('Launch | Make sure w/o Notice'));
      $this->assertNoText(t('warning'), t('Launch | Make sure w/o Warnings'));

      // Make sure listing.
      for ($j = 0; $j < $i + 1; $j++) {
        $this->assertText($add[$j]['name'], t('Make sure w/ Listing: @name', [
          '@name' => $add[$j]['name'],
        ]));
      }
    }

    // Edit an Instance information.
    $edit = $this->createInstanceTestData();
    for ($i = 0; $i < AWS_CLOUD_INSTANCE_REPEAT_COUNT; $i++) {

      $num = $i + 1;

      unset($edit[$i]['cloud_type']);
      unset($edit[$i]['image_id']);
      unset($edit[$i]['min_count']);
      unset($edit[$i]['max_count']);
      unset($edit[$i]['key_pair_name']);
      unset($edit[$i]['is_monitoring']);
      unset($edit[$i]['availability_zone']);
      unset($edit[$i]['security_groups']);
      unset($edit[$i]['instance_type']);
      unset($edit[$i]['kernel_id']);
      unset($edit[$i]['ramdisk_id']);
      unset($edit[$i]['user_data']);

      $this->drupalPostForm("/clouds/aws_cloud/$cloud_context/instance/$num/edit",
                            $edit[$i],
                            t('Save'));

      $this->updateInstanceInMockData($num - 1, $edit[$i]['name']);

      $this->assertResponse(200, t('Edit | HTTP 200:  A New Instance #@num', ['@num' => $num]));
      $this->assertNoText(t('Notice'), t('Edit | Make sure w/o Notice'));
      $this->assertNoText(t('warning'), t('Edit | Make sure w/o Warnings'));
      $this->assertText(t('The AWS Cloud Instance "@name" has been saved.', [
                        '@name' => $edit[$i]['name'],
                      ]),
                        t('Confirm Message') . ': '
                      . t('Edit | The AWS Cloud Instance "@name" has been saved.', [
                        '@name' => $edit[$i]['name'],
                      ]));

      $this->assertResponse(200, t('Edit | HTTP 200: A New Cloud Instance #@num', ['@num' => $num]));
      $this->assertNoText(t('Notice'), t('Edit | Make sure w/o Notice'));
      $this->assertNoText(t('warning'), t('Edit | Make sure w/o Warnings'));
      $this->assertText($edit[$i]['name'], t('Name: @name ', ['@name' => $edit[$i]['name']]));
      $this->assertText(t('The AWS Cloud Instance "@name" has been saved.', [
                        '@name' => $edit[$i]['name'],
                      ]),
                        t('Confirm Message') . ': '
                      . t('Edit | The AWS Cloud Instance "@name" has been saved.', [
                        '@name' => $edit[$i]['name'],
                      ]));

      // Make sure listing.
      $this->drupalGet("/clouds/aws_cloud/$cloud_context/instance");
      $this->assertResponse(200, t('Edit | List | HTTP 200: Instance #@num', ['@num' => $num]));
      $this->assertNoText(t('Notice'), t('Edit | List | Make sure w/o Notice'));
      $this->assertNoText(t('warning'), t('Edit | List | Make sure w/o Warnings'));
      for ($j = 0; $j < $i + 1; $j++) {
        $this->assertText($edit[$i]['name'],
                        t('Edit | List | Make sure w/ Listing: @name', [
                          '@name' => $edit[$i]['name'],
                        ]));
      }
    }

    $options = ['query' => ['destination' => "/clouds/aws_cloud/$cloud_context/instance"]];

    // Terminate Instance.
    for ($i = 0; $i < AWS_CLOUD_INSTANCE_REPEAT_COUNT; $i++) {

      $num = $i + 1;

      $this->drupalGet("/clouds/aws_cloud/$cloud_context/instance/$num/terminate");
      $this->assertResponse(200, t('Terminate: HTTP 200: Instance #@num', ['@num' => $num]));
      $this->assertNoText(t('Notice'), t('Terminate | Make sure w/o Notice'));
      $this->assertNoText(t('warning'), t('Terminate | Make sure w/o Warnings'));

      $this->deleteFirstInstanceInMockData();

      $this->drupalPostForm("/clouds/aws_cloud/$cloud_context/instance/$num/terminate",
                            [],
                            t('Delete | Terminate'),
                            $options);

      $this->assertResponse(200, t('Terminate | HTTP 200: The Cloud Instance #@num', ['@num' => $num]));
      $this->assertNoText(t('Notice'), t('Terminate | Make sure w/o Notice'));
      $this->assertNoText(t('warning'), t('Terminate | Make sure w/o Warnings'));
      $this->assertText($edit[$i]['name'], t('Instance Name: @name ', ['@name' => $edit[$i]['name']]));
      $this->assertText(t('The AWS Cloud Instance "@name" has been terminated.', [
                          '@name' => $edit[$i]['name'],
                        ]),
                        t('Confirm Message') . ': '
                        . t('Terminate | The AWS Cloud Instance "@name" has been terminated.', [
                          '@name' => $edit[$i]['name'],
                        ]));

      // Make sure listing.
      $this->drupalGet("/clouds/aws_cloud/$cloud_context/instance");
      $this->assertResponse(200, t('Terminate | HTTP 200: Instance #@num', ['@num' => $num]));
      $this->assertNoText(t('Notice'), t('Terminate | List | Make sure w/o Notice'));
      $this->assertNoText(t('warning'), t('Terminate | List | Make sure w/o Warnings'));
      for ($j = 0; $j < $i + 1; $j++) { // 3 times
        $this->assertNoText($edit[$i]['name'],
                            t('Terminate | List | Make sure w/ Listing: @name', array(
                            '@name' => $edit[$i]['name'])));
      }
    }
  }

  /**
   *
   */
  private function createInstanceTestData() {
    $data = [];

    for ($i = 0; $i < AWS_CLOUD_INSTANCE_REPEAT_COUNT; $i++) {

      $num = $i + 1;

      // Input Fields.
      $data[$i] = [
        'name'              => "Instance #$num - " . date('Y/m/d - ') . $this->random->name(8, TRUE),
        'image_id'          => 'ami-' . $this->random->name(8, TRUE),
        'min_count'         => $num,
        'max_count'         => $num * 2,
        'key_pair_name'     => "key_pair-$num-" . $this->random->name(8, TRUE),
        'is_monitoring'     => 0,
        'availability_zone' => "us-west-$num",
        'security_groups'   => "security_group-$num-" . $this->random->name(8, TRUE),
        'instance_type'     => "t$num.small",
        'kernel_id'         => 'aki-' . $this->random->name(8, TRUE),
        'ramdisk_id'        => 'ari-' . $this->random->name(8, TRUE),
        'user_data'         => "User Data #$num: " . $this->random->string(64, TRUE),
        'login_username'    => "user-$num" . date('-Y/m/d-') . $this->random->name(8, TRUE),
      ];
    }
    return $data;
  }

  private function createServerTemplate() {
    // create image
    $data = [];
    $image = entity_create('aws_cloud_image', $data);
    $image->set('ami_name', 'image1');
    $image->setImageId('ami-abcdef');
    $image->save();

    $data = [
      'type' => 'aws_cloud',
      'name' => 'test_template1',
    ];

    // create template
    $template = entity_create('cloud_server_template', $data);
    $template->setCloudContext($this->cloud_context);
    $template->field_test_only->value = '1';
    $template->field_max_count->value = 1;
    $template->field_min_count->value = 1;
    $template->field_monitoring->value = '0';
    $template->field_instance_type->value = 't1.micro';
    $template->field_image_id->target_id = 1;

    //field_ssh_key

    $template->save();
  }

  private function addInstanceMockData($name, $key_pair_name) {
    $mock_data = $this->getMockDataFromConfig();
    $vars = $this->getMockDataTemplateVars();
    $vars['name'] = $name;
    $vars['key_name'] = $key_pair_name;
    $instance_mock_data_content = $this->getMockDataFileContent(get_class($this), $vars, '_instance');

    $instance_mock_data = Yaml::decode($instance_mock_data_content);
    $mock_data['DescribeInstances']['Reservations'][0]['Instances'][] = $instance_mock_data;
    $this->updateMockDataToConfig($mock_data);
  }

  private function deleteFirstInstanceInMockData() {
    $mock_data = $this->getMockDataFromConfig();
    $instances = $mock_data['DescribeInstances']['Reservations'][0]['Instances'];
    array_shift($instances);
    $mock_data['DescribeInstances']['Reservations'][0]['Instances'] = $instances;
    $this->updateMockDataToConfig($mock_data);
  }

  private function updateInstanceInMockData($instance_index, $name) {
    $mock_data = $this->getMockDataFromConfig();
    $mock_data['DescribeInstances']['Reservations'][0]['Instances'][$instance_index]['Tags'][0]['Value'] = $name;
    $this->updateMockDataToConfig($mock_data);
  }
}
